#include "stdafx.h"

using namespace std;

#define CHART_DEFAULT_WIDTH 350
#define CHART_DEFAULT_HEIGHT 200
#define CHART_PIE_WIDTH 300
#define CHART_PIE_HEIGHT 230

bool CTeeChart_MicroDemo::GenerateChart(EChartType type, LPCTSTR sOutFile, CWnd* pParentWindow)
{
  CTChart tChart;
  int nWidth  = CHART_DEFAULT_WIDTH;
  int nHeight = CHART_DEFAULT_HEIGHT;

  if (type == eCHART_Pie || type == eCHART_Donut)
  {
    nWidth = CHART_PIE_WIDTH;
    nHeight = CHART_PIE_HEIGHT;
  }

  if (!tChart.Create (NULL, NULL, 0, CRect (0,0,nWidth,nHeight), pParentWindow, 0, NULL))
  {
    return false;
  }

  InitChart(tChart, type);

  SetChart(tChart, type);

  //// Due to a "bug" we have to clear the flag
  //// otherwise chart may show an error even
  //// it the error does not "belong" to it
  //_clearfp();

  tChart.GetExport().GetAsPNG().SaveToFile(sOutFile);

  tChart.DestroyWindow();

  return true;
}

bool CTeeChart_MicroDemo::InitChart(CTChart& rTChart, EChartType type)
{
  //
  // Set Chart styles
  //
  rTChart.GetPanel().SetColor(COLORREF_White);
  rTChart.GetPanel().SetBevelInner(0);
  rTChart.GetPanel().SetBevelOuter(0);
  rTChart.GetPanel().GetGradient().SetVisible(FALSE);
  rTChart.GetPanel().GetGradient().GetSubGradient().SetVisible(FALSE);
  rTChart.GetLegend().SetVisible(FALSE);
  rTChart.GetHeader().SetVisible(FALSE);
  rTChart.GetFooter().SetVisible(FALSE);
  rTChart.GetAspect().SetView3D(FALSE);
  rTChart.GetAspect().GetGDIPlus().SetActive(FALSE);

  rTChart.GetWalls().SetVisible(TRUE);
  rTChart.GetWalls().GetBack().GetPen().SetVisible(FALSE);
  rTChart.SetClipPoints(TRUE);		

  switch (type)
  {
  case eCHART_Pie :
  case eCHART_Donut :
    //rTChart.GetAspect().SetElevation(320);
    //rTChart.GetAspect().SetChart3DPercent(50);
    break;

  case eCHART_Line :
    {
      CAxis		  cAxis;

      // Bottom axis
      cAxis = rTChart.GetAxis().GetBottom();
      cAxis.SetExactDateTime(FALSE);
      cAxis.SetIncrement(COleDateTimeSpan(0, 0, 0, 10));	// 10 seconds

      cAxis.GetLabels().SetStyle(talAuto);
      cAxis.SetAutomatic(TRUE);
      cAxis.GetLabels().SetDateTimeFormat(_T("hh:mm:ss"));
      cAxis.GetAxisPen().SetWidth(1);
      cAxis.GetLabels().SetSeparation(30);
      cAxis.SetMinorTickCount(0);
      cAxis.SetTickLength(0);
      cAxis.GetGridPen().SetVisible(FALSE); // GRid Border

      // Left axis
      cAxis = rTChart.GetAxis().GetLeft ();
      cAxis.SetTitleSize(1); // 0 for automatic requires too much space
      cAxis.GetLabels().SetStyle(talAuto);
      cAxis.SetAutomatic(TRUE);
      cAxis.GetLabels().SetDateTimeFormat(_T("dd.mm.yyyy hh:mm:ss"));
      cAxis.GetAxisPen().SetWidth(1);
      // we ant the charts left axis start at same position for all charts
      cAxis.GetLabels().SetSize (32);
      // set start position so maximum data is in the chart and not clipped  
      //cAxis.SetStartPosition(4);
      // set axis Minimum to 0 and Maximum to 100
      cAxis.SetAutomaticMinimum(FALSE);  
      cAxis.SetAutomaticMaximum(FALSE);  
      cAxis.SetMinMax(0.0, 100.0);

      cAxis.SetMinorTickCount(0);
      cAxis.SetTickLength(0);
      cAxis.GetTicks().SetVisible(FALSE);
      cAxis.GetLabels().SetOnAxis(TRUE);
      cAxis.GetAxisPen().SetVisible(FALSE); // Axis Border
      //cAxis.GetTitle().SetCaption (_T(LANGUAGE_LOAD_STRING(IDS_WBGENERATECHART_35135,"active users & errors")));
      //cAxis.GetGridPen().SetStyle(psSmallDot);
      cAxis.GetGridPen().SetVisible(FALSE);

      double fStartTime = (double)time(0);
      double fEndTime = fStartTime + 60000;

      if (fStartTime != 0)
      {
        // Set Minimum / Maximum of xAxis for all charts
        // to the tsdfile MinMax Time    
        cAxis = rTChart.GetAxis().GetBottom();
        cAxis.SetAutomaticMaximum(FALSE);
        cAxis.SetAutomaticMinimum(FALSE);
        cAxis.SetMinMax(fStartTime, fEndTime);
      }

      AddGridBand(rTChart, COLORS_GRIDBAND1, COLORS_GRIDBAND2, COLORS_GRIDBAND_ALPHA);

      break;
    }
  case eCHART_Bar :
    {
      // Bottom axis
      CAxis cAxis = rTChart.GetAxis().GetBottom();
      cAxis.SetIncrement(10);
      cAxis.GetLabels().SetStyle(talAuto);
      cAxis.SetAutomatic(TRUE);
      cAxis.GetLabels().SetDateTimeFormat(_T("hh:mm:ss"));
      cAxis.GetAxisPen().SetWidth(1);
      cAxis.GetLabels().SetSeparation(30);
      cAxis.SetMinorTickCount(0);
      cAxis.SetTickLength(0);
      cAxis.GetGridPen().SetVisible(FALSE); // GRid Border
      cAxis.GetTitle().SetCaption(_T("VUsers"));

      // Left axis
      cAxis = rTChart.GetAxis().GetLeft ();
      cAxis.SetTitleSize(0);
      cAxis.GetLabels().SetStyle(talAuto);
      cAxis.SetAutomatic(TRUE);
      cAxis.GetLabels().SetDateTimeFormat(_T("dd.mm.yyyy hh:mm:ss"));
      cAxis.GetAxisPen().SetWidth(1);
      // we ant the charts left axis start at same position for all charts
      cAxis.GetLabels().SetSize (32);
      // set start position so maximum data is in the chart and not clipped  
      cAxis.SetStartPosition(4);
      // set axis Minimum to 0
      cAxis.SetAutomaticMinimum(FALSE);  
      cAxis.SetMinimum(0.0);  
      cAxis.SetLogarithmic(TRUE);
      cAxis.SetMinorTickCount(0);
      cAxis.SetTickLength(0);
      cAxis.GetTicks().SetVisible(FALSE);
      cAxis.GetLabels().SetOnAxis(FALSE);
      cAxis.GetAxisPen().SetVisible(FALSE); // Axis Border
      cAxis.GetTitle().SetCaption(_T("kB/s"));
      cAxis.GetGridPen().SetStyle(psSmallDot);

      break;
    }
  }
  return TRUE;
}

bool CTeeChart_MicroDemo::SetChart(CTChart& rTChart, EChartType type)
{
  switch (type)
  {
  case eCHART_Pie :
  case eCHART_Donut :
    {
      if (type == eCHART_Donut)
      {
        rTChart.AddSeries(scDonut);
      }
      else
      {
        rTChart.AddSeries(scPie);
      }
      long count = rTChart.GetSeriesCount();
      CSeries	ChartSeries = rTChart.Series(count - 1);
      ChartSeries.Clear();
      ChartSeries.GetMarks().SetStyle(smsValue);
      ChartSeries.GetMarks().SetVisible(TRUE);
      ChartSeries.GetMarks().SetTransparent(TRUE);
      ChartSeries.GetMarks().GetArrow().SetVisible(FALSE);
      ChartSeries.GetMarks().GetFrame().SetColor(RGB(0x80,0x80,0x80));
      ChartSeries.GetMarks().GetFrame().SetVisible(FALSE);
      ChartSeries.GetMarks().SetArrowLength(0);
      if (type == eCHART_Donut)
      {
        ChartSeries.GetAsDonut().SetDonutPercent(40);
        ChartSeries.GetAsDonut().SetCircled(TRUE);
        ChartSeries.GetAsDonut().GetShadow().SetVisible(FALSE);
        ChartSeries.GetAsDonut().SetCircleBackColor(RGB(255,255,255));
        ChartSeries.GetAsDonut().SetDark3D(FALSE);
        //ChartSeries.GetAsDonut().SetExplodeBiggest(5);

        CPen1 piePen = ChartSeries.GetAsDonut().GetPiePen();
        piePen.SetColor(RGB(255,255,255));
        piePen.SetWidth(2);
      }
      else
      {
        ChartSeries.GetAsPie().SetCircled(TRUE);
        ChartSeries.GetAsPie().SetCircleBackColor(RGB(255,255,255));
        ChartSeries.GetAsPie().SetDark3D(FALSE);
      }
      ChartSeries.SetActive(TRUE);
      ChartSeries.Add(0.01, _T("Pie1"), COLORREF_Green);
      ChartSeries.Add(0.05, _T("Pie2"), COLORREF_Blue);
      ChartSeries.Add(0.07, _T("Pie3"), COLORREF_Magenta);
      ChartSeries.Add(10.0, _T("Pie4"), COLORREF_Red);
      ChartSeries.Add(20.0, _T("Pie5"), COLORREF_Yellow);
    }
    break;

  case eCHART_Line :
    {
      rTChart.AddSeries(scLine); 
      long count = rTChart.GetSeriesCount();

      CSeries	ChartSeries = rTChart.Series(count - 1);
      ChartSeries.Clear();
      ChartSeries.GetMarks().SetStyle(smsValue);
      ChartSeries.SetValueFormat(_T("0.00"));
      ChartSeries.GetMarks().SetVisible(FALSE);
      ChartSeries.SetActive(TRUE);

      ChartSeries.SetCursor(-21);  // Hand MouseCursor   
      ChartSeries.SetColor(COLORREF_Blue);

      ChartSeries.GetXValues().SetDateTime(TRUE);
      ChartSeries.GetAsLine().GetLinePen().SetStyle(0);
      ChartSeries.GetAsLine().GetLinePen().SetWidth(2);
      ChartSeries.GetAsLine().SetStairs(TRUE);
      ChartSeries.GetAsLine().SetInvertedStairs(TRUE);

      double currentTime = (double)time(0);
      ChartSeries.AddXY(currentTime, 10, NULL, COLORREF_Blue);
      ChartSeries.AddXY(currentTime + 10000, 10, NULL, COLORREF_Blue);
      ChartSeries.AddXY(currentTime + 20000, 10, NULL, COLORREF_Blue);
      ChartSeries.AddXY(currentTime + 30000, 10, NULL, COLORREF_Blue);
      ChartSeries.AddXY(currentTime + 40000, 10, NULL, COLORREF_Blue);

      AddColorBand(rTChart, currentTime + 10000, currentTime + 30000, COLORREF_LightGreen, COLORS_GRIDBAND_ALPHA);
    }
    break;
  case eCHART_Bar :
    {
        rTChart.AddSeries(scBar); 
        long count = rTChart.GetSeriesCount();
        CSeries	ChartSeries = rTChart.Series(count - 1);
        ChartSeries.Clear();
        ChartSeries.GetMarks().SetStyle(smsValue);
        ChartSeries.SetValueFormat(_T("0.00"));
        ChartSeries.GetMarks().SetTransparent(TRUE);
        ChartSeries.GetMarks().SetVisible(TRUE);
        ChartSeries.GetMarks().SetBackColor(RGB(0xDC,0xDC,0xDC));
        ChartSeries.GetMarks().GetArrow().SetVisible(FALSE);
        ChartSeries.GetMarks().GetFrame().SetColor(RGB(0x80,0x80,0x80));
        ChartSeries.GetMarks().GetFrame().SetVisible(FALSE);
        ChartSeries.GetMarks().SetArrowLength(2);
        ChartSeries.GetMarks().SetClip(TRUE);
        ChartSeries.SetActive(TRUE);

        ChartSeries.SetCursor(-21);  // Hand MouseCursor   
        ChartSeries.SetColor(COLORREF_Green);

        ChartSeries.GetXValues().SetDateTime(FALSE);
        CBarSeries s = ChartSeries.GetAsBar();
        s.SetMultiBar(mbSide);

        ChartSeries.Add(0.5, _T("1"), COLORREF_Green);
        ChartSeries.Add(5.0, _T("10"), COLORREF_Green);
        ChartSeries.Add(50.0, _T("100"), COLORREF_Green);
    } // End Case eCHART_Line

    break;
  } // End switch

  return TRUE;
}

bool CTeeChart_MicroDemo::AddColorBand(CTChart& rTChart, double dStartValue, double dEndValue, COLORREF color, int alpha)
{
  if (dStartValue == dEndValue)
  {
    return false;
  }

  CToolList lTools = rTChart.GetTools();
  long index = lTools.Add(tcColorband);
  CTools tools = lTools.GetItems(index);
  tools.SetActive(TRUE);
  CColorBandTool colorBandTool = tools.GetAsColorband();

  COleVariant objVariant( (short)atBottom );
  VARIANT variantBottomAxis = objVariant.Detach();

  colorBandTool.SetAxis(variantBottomAxis);
  colorBandTool.SetTransparency(alpha);
  colorBandTool.SetStartValue(dStartValue);
  colorBandTool.SetEndValue(dEndValue);
  colorBandTool.SetColor(color);
  colorBandTool.SetDrawBehind(TRUE);

  return true;
}

bool CTeeChart_MicroDemo::AddGridBand(CTChart& rTChart, COLORREF colorBand1, COLORREF colorBand2, int alpha)
{
  CToolList lTools = rTChart.GetTools();
  long index = lTools.Add(tcGridBand);
  CTools tools = lTools.GetItems(index);
  tools.SetActive(TRUE);
  CGridBandTool gridBandTool = tools.GetAsGridBand();

  COleVariant objVariant( (short)atLeft );
  VARIANT variantLeftAxis = objVariant.Detach();

  gridBandTool.SetAxis(variantLeftAxis);
  CGridBandBrush gridBandBrush1 = gridBandTool.GetBand1();
  gridBandBrush1.SetColor(colorBand1);
  gridBandBrush1.SetTransparency(alpha);
  CGridBandBrush gridBandBrush2 = gridBandTool.GetBand2();
  gridBandBrush2.SetColor(colorBand2);
  gridBandBrush2.SetTransparency(alpha);

  return true;
}
